﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Drawing2D;
using System.Collections.Specialized;

namespace PictureEditor
{    
    /* 
     * This class handle all images editor
     * such as resize, rotate, flip
     */
    public class ImageEditor
    {
        /** original file member variable **/
        private string m_originalFile = string.Empty;
        
        /** property of member variable original file **/
        public string originalFile
        {
            get { return m_originalFile; }
            set { m_originalFile = value; }
        }

        /*
         * Resize folder function 
         * @param inputFolder is folder that contain all jpeg files that we want to input
         * @param destinationFolder is destination Folder that we want to put
         * @param desiredSize is size that we desire
         */
        public bool resizeFolder(string inputFolder, string destinationFolder,
                                 Size desiredSize)
        {
            bool result = false;

            //if input directory is not exist, then return false
            if (!Directory.Exists(inputFolder)) return result;

            try
            {
                //find jpeg files in input directory
                string[] fileList = Directory.GetFiles(inputFolder);
                StringCollection jpegfiles = new StringCollection();

                //if no files in the list
                if (fileList.Count() == 0) return result;

                //browse all files to look jpeg files
                for (int i = 0; i < fileList.Count(); ++i)
                {
                    //get file info
                    FileInfo f = new FileInfo(fileList[i]);

                    //find jpeg file and put it in the list
                    if ((f.Extension == ".JPEG") ||
                        (f.Extension == ".jpeg")||
                        (f.Extension == ".jpg")||
                        (f.Extension == ".JPG"))
                    {
                        jpegfiles.Add(f.FullName);
                    }
                } 

                //check if jpeg file is not in the directory return 
                if (jpegfiles.Count == 0) return result;

                //check if destination folder is not exist, create it
                if (!Directory.Exists(destinationFolder))
                {
                    Directory.CreateDirectory(destinationFolder);
                }

                // browse jpeg files
                for (int i = 0; i < jpegfiles.Count; ++i)
                {
                    FileInfo f = new FileInfo(jpegfiles[i]);
                    string destination = string.Empty;
                    destination = destinationFolder + "\\" + f.Name;
                    
                    //resize image,if not success tell user
                    resizeImage(jpegfiles[i],destination, desiredSize);
                }
                result = true;

            }
            catch (IOException ioe)
            {
                handleException("ImageEditor", "resizeFolder", ioe.Message);
            }
            
            return result;
        }

        /* 
         * resize image function
         * @param destinationFile is new file destination
         * @param desiredSize is size that we want to resize
         */
        public bool resizeImage(string destinationFile,Size desiredSize)
        {
            return resizeImage(destinationFile,
                               m_originalFile, desiredSize);
        }

        public bool resizeImage(string file_name,
                                string destinationFile, Size desiredSize)
        {
            bool result = false;

            //if destination file is exist, then delete it 
            if (File.Exists(destinationFile)) File.Delete(destinationFile);

            //if you cannot find the file, return false
            if (!File.Exists(file_name)) return result;
            try
            {
                //load image
                Image i = Image.FromFile(file_name);

                //get source image size
                int sourceWitdh = i.Width;
                int sourceHeight = i.Height;

                float nPercent = 0, nPercentW = 0, nPercentH = 0;

                //calculate the percentage of new image
                nPercentW = (float)desiredSize.Width / (float)sourceWitdh;
                nPercentH = (float)desiredSize.Height / (float)sourceHeight;

                if (nPercentH < nPercentW) nPercent = nPercentH;
                else nPercent = nPercentW;

                //calculate the destination size
                int destWidth = (int)(sourceWitdh * nPercent);
                int destHeight = (int)(sourceHeight * nPercent);

                //create new image with new size
                Bitmap b = new Bitmap(destWidth, destHeight);
                Graphics g = Graphics.FromImage((Image)b);
                g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                g.DrawImage(i, 0, 0, destWidth, destHeight);

                //save to jpeg
                Image newImage = (Image)b;
                newImage.Save(destinationFile);
                //dispose image
                g.Dispose();
                i.Dispose();

                result = true;
            }

            catch (ArgumentNullException ane)
            {
                handleException("ImageEditor", "resizeImage", ane.Message);
            }
            catch (ArgumentException ae)
            {
                handleException("ImageEditor", "resizeImage", ae.Message);
            }
            catch (OutOfMemoryException oe)
            {
                handleException("ImageEditor", "resizeImage", oe.Message);
            }
            catch (Exception ex)
            {
                handleException("ImageEditor", "resizeImage", ex.Message);
            }
            return result;
        }

        /*
         * Rotate image function
         * @param angle is the angle that we want to rotate
         * @param new_file_name is 
         */
        public bool rotateImage(float angle,ref string new_file_name)
        {
            return rotateImage(m_originalFile, angle,ref new_file_name);
        }
       
        /*
         * Rotate image function
         * @param file_name is source_file that we want to rotate
         * @param angle is angle that we want to rotate
         * @param new_file_name is temporary file
         */
        public bool rotateImage(string file_name, float angle,
                                ref string new_file_name)
        {
            bool result = false;            

            //if file not exist return false
            if (!File.Exists(file_name)) return result;

            //create new file_name
            FileInfo f = new FileInfo(file_name);
            new_file_name = f.Directory.ToString() + "\\test" + f.Name;

            //if destination file is already exist delete it
            if (File.Exists(new_file_name)) File.Delete(new_file_name);

            try
            {
                //load image from file
                Bitmap b = (Bitmap)Image.FromFile(file_name);
                Graphics g = Graphics.FromImage(b);

                //check rotation if 90,180,270 do flip
                if (angle == 90)
                {
                    b.RotateFlip(RotateFlipType.Rotate90FlipNone);
                }
                else if (angle == 180)
                {
                    b.RotateFlip(RotateFlipType.Rotate180FlipNone);
                }
                else if (angle == 270)
                {
                    b.RotateFlip(RotateFlipType.Rotate270FlipNone);
                }
                else
                {
                    //move rotation point to center of image
                    g.TranslateTransform((float)b.Width / 2,
                                         (float)b.Height / 2);
                    //rotate
                    g.RotateTransform(angle, MatrixOrder.Append);

                    //move image back
                    g.TranslateTransform(-(float)b.Width / 2,
                                         -(float)b.Height / 2);

                    //draw passed in image onto graphics object
                    g.DrawImage(b, new Point(0, 0));
                }

                b.Save(new_file_name);

                //dispose object
                g.Dispose();
                b.Dispose();

                result = true;
            }

            catch (ArgumentException ae)
            {
                handleException("ImageEditor", "rotateImage", ae.Message);
            }
            catch (IOException ie)
            {
                handleException("ImageEditor", "rotateImage", ie.Message);
            }
            catch (Exception ex)
            {
                handleException("ImageEditor", "rotateImage", ex.Message);
            }
            return result;
        }

        /**
         * Flip Image function
         * @param file_name is filename that we want to flip
         * @param xy is type of flip that we want x / y
         * @param new_file_name is the result of the flip image
         */
        public bool flipImage(string file_name, string XY, 
                              ref string new_file_name)
        {
            bool result = false;

            //if file not exist return false
            if (!File.Exists(file_name)) return result;

            //create new file_name
            FileInfo f = new FileInfo(file_name);
            new_file_name = f.Directory.ToString() + "\\test" + f.Name;

            //if file is already exist, delete file first
            if (File.Exists(new_file_name)) File.Delete(new_file_name);

            try
            {
                //load image from file
                Bitmap b = (Bitmap)Image.FromFile(file_name);
                Graphics g = Graphics.FromImage(b);

                //check flipping type
                if (XY.Equals("X"))
                {
                    b.RotateFlip(RotateFlipType.RotateNoneFlipX);
                }
                else
                {
                    b.RotateFlip(RotateFlipType.RotateNoneFlipY);
                }

                b.Save(new_file_name);

                //dispose object
                g.Dispose();
                b.Dispose();

                result = true;
            }

            catch (ArgumentException ae)
            {
                handleException("ImageEditor", "flipImage", ae.Message);
            }
            catch (IOException ie)
            {
                handleException("ImageEditor", "flipImage", ie.Message);
            }
            catch (Exception ex)
            {
                handleException("ImageEditor", "flipImage", ex.Message);
            }
            return result;

        }      

        /** 
         * What we gonna do when exception occurs 
         * @  
         */
        public static void handleException(string className, string functionName,
                                           string errorType)
        {
            string msg = "Threw error in " + className + "::" + functionName +
                         " with error : " + errorType;

            MessageBox.Show(msg);
        }

        
    }
}
